#ifndef MD_INTEGRATORNOSEHOOVER_H
#define MD_INTEGRATORNOSEHOOVER_H

/*
    Copyright (C) 2010  Thomas Schrøder

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    LICENSE.txt file for license details.
*/

#include "md_Integrator.h"
#include "md_sample.h"

class IntegratorNoseHoover : public Integrator{
  
 private:
  void AllocateThermostatState(int nb, float initialThermostatState);
  void FreeAllocatedMemory();

  Sample* S; // The current sample.
  
  float* h_thermostatState; // CPU.
  float* d_thermostatState; // GPU.
  
  float* d_thermostatKineticEnergy; // Array for summation of kinetic energy.
  float4* d_particleMomentum; // Array for summation of total momentum.
  
  float timeStep; 
  float thermostatRelaxationTime; // The user chosen relaxation time.
  float targetTemperature; 
  float thermostatMass; 

  int numberOfThermostats; // For parallel summation.

 public:
  IntegratorNoseHoover(Sample* S);	
  ~IntegratorNoseHoover();

  // Class methods.
  void Integrate();
  void AllocateThermostatState();
  void UpdateThermostatState();
  
  // Set methods.
  void SetTimeStep(float dt);
  void SetRelaxationTime(float tau);
  void SetTargetTemperature(float T);
  void SetThermostatState(float Ps);
  void SetMomentumToZero();

  // Get methods.
  float GetTimeStep();
  float GetRelaxationTime();
  float GetTargetTemperature();
  float GetThermostatState();
  double GetKineticEnergy();
  float4 GetTotalMomentum();
};

#endif // MD_INTEGRATORNOSEHOOVER_H
