#ifndef MD_PARTICLEDATA_H
#define MD_PARTICLEDATA_H

/*
    Copyright (C) 2010  Thomas Schrøder

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    LICENSE.txt file for license details.
*/

#include "rumd_base.h"
#include "rumd_technical.h"
#include <vector_types.h>

class ParticleData {
  
public:
  ParticleData( unsigned int set_particles_per_block, 
		unsigned int set_threads_per_particle );
  
  ~ParticleData();
  

  void SetNumberOfParticles(int num_part);

  void AllocateParticles(int nvp);

  //NB void AllocateBlocksParticles(int nb_vp);
  
  void FreeParticles();

  //NB  void FreeBlocksParticles();

  // Functions for transferring data to the device

  void CopyPosToDevice();

  void CopyVelToDevice();

  void CopyForToDevice();

  void CopyImagesToDevice();

  void CopyConfToDevice(); // all of them together

  void CopyConstraintsToDevice();

  // Functions for transferring data from the device

  void CopyPosFromDevice();
  
  void CopyVelFromDevice();

  void CopyForFromDevice();
  
  void CopyVirFromDevice();
  
  void CopyImagesFromDevice();

  void CopyStressFromDevice();
  
  void CopyConfFromDevice(); // all of them together


  //protected:

  unsigned int numParticles;
  //unsigned int numTypes;
  unsigned int numBlocks; 
  unsigned int numVirtualParticles; // number of 'particles' assigned threads (>=numParticles)
  int particles_per_block;
  int threads_per_particle;

  // for managing dynamic allocation
  int allocatedNumParticles;
  //NB int allocatedNumBlocksParticles;
	
  // ===== Configuration on host =====

  float4* h_r;     //=={x, y, z, type}=====================
  float4* h_v;     //=={vx, vy, vz, 1/mass}== t+dt/2 ======
  float4* h_vb;    //=={vx, vy, vz, 1/mass}== t-dt/2 ======
  float4* h_f;     //=={fx, fy, fz, u}=====================
  float4* h_w;     //=={w_vdw_mol, sxz, sxy, w_vdw_atom}===
  float4* h_im;    //=={imx, imy, imz, 0}==================
  float4* h_sts;   //=={sxx, syy, szz, syz}================
  unsigned int*    h_Type;    //  Type (on cpu)
  
  //NB NBTYPEv* h_CNbList; // The elimination bit-pattern.   

  // ===== Configuration on device =====

  float4* d_r; 
  float4* d_v; 
  float4* d_vb;
  float4* d_f;
  float4* d_w;
  float4* d_im;
  float4* d_sts;
  
  //NB  NBTYPEv *NbList;
  //NB  NBTYPEv* d_CNbList;   
};


#endif // MD_PARTICLEDATA_H
