#ifndef MD_KERNEL_H
#define MD_KERNEL_H

/*
    Copyright (C) 2010  Thomas Schrøder

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    LICENSE.txt file for license details.
*/

#include "md_kernel_forces.h"

///////////////////////////////////////////////////////////////////////////
// This header defines global kernel implementations in the different files.
///////////////////////////////////////////////////////////////////////////

////////////////////// md_kernel_nblist.cu //////////////////////

// __device__ float calc_dist2( float4 my_r, float4 rj, float L, float invL );
__global__ void md_update_nblist0( unsigned int num_part, float4* r, float4* im, float* params, float* L, float* invL, NBTYPEv *NbList, int* cell_list, int* check );
__global__ void md_update_nblist( unsigned int num_part, float4* r, float4* im, float* params, float* L, float* invL, NBTYPEv *NbList, int* cell_list, int *check );
// __device__ void update_nblist( unsigned int num_part, float4 my_r, float4 *r, float* params, float L, float invL, NBTYPEv *NbList );

////////////////////// md_IntegratorNoseHoover.cu //////////////////////

__global__ void integrateNoseHooverAlgorithm( unsigned numParticles, float4* position, float4* velocity, float4* force, float4* image, 
					      float* boxL, float* invboxL, float* thermostatState, int* updateNbList, float timeStep, float nbListSkin );
__global__ void partialSummationKineticEnergy( unsigned int numParticles, float4* velocity, float* thermostatKineticEnergy );
__global__ void updateNoseHooverThermostatState( float* thermostatKineticEnergy, float* thermostatState, float thermostatMass, 
						 float targetTemperature, int degreesOfFreedom, float timeStep );
__global__ void partialSummationMomentum( unsigned int numParticles, float4* velocity, float4* particleMomentum );
__global__ void zeroTotalMomentum( unsigned int numParticles, float4* velocity, float4* particleMomentum );

////////////////////// md_IntegratorNVU.cu //////////////////////

__global__ void integrateNVUAlgorithm( unsigned numParticles, float4* position, float4* velocity, float4* force, float4* image, float* boxL, float* invboxL, 
				       int* updateNbList, float* constraintForceCorrection, float* diplacementLengthCorrection, float nbListSkin );
__global__ void partialSummationConstraintForce( unsigned numParticles, float4* velocity, float4* force, float* constraintForceNumerator, float* constraintForceDenominator );
__global__ void calculateConstraintForce( float* constraintForceNumerator,  float* constraintForceDenominator, float* constraintForceCorrection, 
					  float* totalConstraintForceDenominator );
__global__ void partialSummationNumericalCorrection( unsigned numParticles, float4* velocity, float4* force, float* constraintForce, float* constraintForceDenominator,  
						     float* constraintForceCorrection, float* potentialEnergyCorrection, float* displacementLengthCorrection, 
						     float* previousPotentialEnergy, float targetPotentialEnergy, float displacementLength, float potentialEnergyFactor );
__global__ void calculateNumericalCorrection( float* potentialEnergyArray, float* displacementLengthArray, float* displacementLengthCorrection,
					      float* previousPotentialEnergy, float displacementLength );
__global__ void partialSummationVelocity( unsigned numParticles, float4* velocity, float4* particleVelocity );
__global__ void zeroSystemVelocity( unsigned numParticles, float4* velocity, float4* particleVelocity );

//////////////////////////////////////////////////////////////////////
  
#endif // MD_KERNEL_H
